use crate::algorithm::Algorithm;
use crate::game::Game;

pub struct AlphaBeta {}

impl Algorithm for AlphaBeta {
    fn compute<T: Game>(game: &T, depth: i32) -> Option<T::Move> {
        let mut cl = game.clone();
        let mut alpha = T::SCORE_MIN;
        let beta = T::SCORE_MAX;
        if depth == 0 || game.is_game_over() {
            None
        } else {
            let mut best_move = None;
            for mov in game.possible_moves() {
                let undo = cl.do_move(mov);
                let result = -Self::alpha_beta_rec(&mut cl, depth - 1, -beta, -alpha);
                cl.undo_move(undo);
                if result > alpha {
                    best_move = Some(mov);
                    alpha = result;
                    if result == -game.score_if_win_next_round() {
                        // with this move we have won! We cannot do it better!!
                        break;
                    }
                }
            }
            best_move
        }
    }
}

impl AlphaBeta {
    fn alpha_beta_rec<T: Game>(game: &mut T, depth: i32, mut alpha: i32, beta: i32) -> i32 {
        if depth == 0 || game.is_game_over() {
            return game.get_score();
        }
        for mov in game.possible_moves() {
            let undo = game.do_move(mov);
            let result = -Self::alpha_beta_rec(game, depth - 1, -beta, -alpha);
            game.undo_move(undo);

            if result >= beta {
                // Pruning:
                // This move would be better for the current player,
                // but the other player won't allow him to make this move.
                // So recording this move isn't worth it.
                return beta;
            }
            if result > alpha {
                alpha = result;
                if result == -game.score_if_win_next_round() {
                    // with this move we have won! We cannot do it better!!
                    break;
                }
            }
        }
        alpha
    }
}
